/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2004  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "prthread.h"
#include <ace/OS_NS_Thread.h>
#include <errno.h>

PRThread* PR_GetCurrentThread()
{
	// libjs expects thread ids not to be 0
	// but always have the least significant bit clear.
	// errno fits perfectly here.
	return reinterpret_cast<PRThread*>(&errno);
}

PRStatus PR_NewThreadPrivateIndex(
	PRUintn* newIndex, PRThreadPrivateDTOR destructor)
{
	ACE_thread_key_t key;
	if (ACE_OS::thr_keycreate(&key, destructor) == -1) {
		return PR_FAILURE;
	} else {
		*newIndex = static_cast<PRUintn>(key);
		return PR_SUCCESS;
	}
}


PRStatus PR_SetThreadPrivate(PRUintn tpdIndex, void* priv)
{
	ACE_thread_key_t key = static_cast<ACE_thread_key_t>(tpdIndex);
	return static_cast<PRStatus>(ACE_OS::thr_setspecific(key, priv));
}

void* PR_GetThreadPrivate(PRUintn tpdIndex)
{
	void* data = 0;
	ACE_thread_key_t key = static_cast<ACE_thread_key_t>(tpdIndex);
	ACE_OS::thr_getspecific(key, &data);
	return data;
}
