/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2004  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef ADSUSPECT_H_
#define ADSUSPECT_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "RefCountable.h"
#include "RefCounter.h"
#include <memory>
#include <iosfwd>

class URI;
class HeuristicScore;
class SplittableBuffer;
class BString;

class AdSuspect : public RefCountable<RefCounter<ACE_NULL_SYNCH> >
{
public:
	virtual ~AdSuspect() {}
	
	virtual int getWidth() const = 0;  // returns -1 if unknown
	
	virtual int getHeight() const = 0; // returns -1 if unknown
	
	virtual URI const& getLocation() const = 0;
	
	virtual bool isLocationAbsolute() const = 0;
	
	virtual HeuristicScore const& getScore() const = 0;
	
	virtual void writeSubstitution(std::ostream& strm) const = 0;
	
	virtual BString getSubstitutionURL() const = 0;
	
	std::auto_ptr<SplittableBuffer> getHtmlSubstitution() const;
	
	std::auto_ptr<SplittableBuffer> getJsSubstitution() const;
	
	static BString getSubstitutionURL(
		char const* prefix, URI const& orig_url, 
		int width, int height);
	
	static BString getAnalyzeURL(
		URI const& orig_url, bool ignore_size);
	
	static BString getAnalyzeURL(
		URI const& orig_url, HeuristicScore const& score);
};

#endif
