/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DECOMPRESSORRESPONSEFILTER_H_
#define DECOMPRESSORRESPONSEFILTER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "ResponseFilterBase.h"
#include "NonCopyable.h"
#include <string>
#include <memory>

class AbstractDecompressor;
class RequestStatus;
class HttpResponseMetadata;
class HttpHeader;
class BString;

class DecompressorResponseFilter : public ResponseFilterBase
{
	DECLARE_NON_COPYABLE(DecompressorResponseFilter)
public:
	DecompressorResponseFilter(ResponseFilterChain& filter_chain);
	
	virtual ~DecompressorResponseFilter();
	
	virtual void processMetadata(
		RequestStatus& status,
		std::auto_ptr<HttpResponseMetadata> metadata);
	
	virtual void processBodyData(
		RequestStatus& status,
		SplittableBuffer& data, bool eof);
	
	static bool isSupported(BString const& compression);
private:
	enum { OUTPUT_LIMIT = 16384 };
	
	typedef std::auto_ptr<AbstractDecompressor> (*DescriptorConstructor)();
	
	struct DecompressorDescriptor
	{
		char const* name;
		DescriptorConstructor constructor;
	};
	
	static DecompressorDescriptor const* findDescriptor(BString const& decompressor);
	
	static std::auto_ptr<AbstractDecompressor> createGzipDecompressor();
	
	static std::auto_ptr<AbstractDecompressor> createDeflateDecompressor();
	
	void setupDecompressor(HttpResponseMetadata& metadata, HttpHeader* hdr);
	
	static DecompressorDescriptor const m_sAvailableDecompressors[];
	std::auto_ptr<AbstractDecompressor> m_ptrDecompressor;
	bool m_isEOF;
};

#endif
