/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2004  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef FLASHADSUSPECT_H_
#define FLASHADSUSPECT_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "AdSuspect.h"
#include "FlashHeuristicScore.h"
#include "URI.h"
#include "BString.h"
#include <memory>

class FlashAdSuspect : public AdSuspect
{
public:
	FlashAdSuspect(int width, int height,
		URI const& location, bool location_absolute,
		FlashHeuristicScore const& score);
	
	virtual ~FlashAdSuspect();
	
	virtual int getWidth() const;
	
	virtual int getHeight() const;
	
	virtual URI const& getLocation() const;
	
	virtual bool isLocationAbsolute() const;
	
	virtual HeuristicScore const& getScore() const;
	
	virtual BString getSubstitutionURL() const;
	
	virtual void writeSubstitution(std::ostream& strm) const;
	
	static BString getSubstitutionURL(
		URI const& orig_url, int width, int height);
private:
	int m_width;  //
	int m_height; // -1 if undefined
	URI m_location;
	bool m_isLocationAbsolute;
	FlashHeuristicScore m_score;
};


inline
FlashAdSuspect::FlashAdSuspect(
	int width, int height,
	URI const& location, bool location_absolute,
	FlashHeuristicScore const& score)
:	m_width(width),
	m_height(height),
	m_location(location),
	m_isLocationAbsolute(location_absolute),
	m_score(score)
{
}

inline BString
FlashAdSuspect::getSubstitutionURL(
	URI const& orig_url, int width, int height)
{
	return AdSuspect::getSubstitutionURL("bf-sf-", orig_url, width, height);
}

#endif
