/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2007 Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IMAGEHEURISTICSCORE_H_
#define IMAGEHEURISTICSCORE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "HeuristicScore.h"

class URI;
class BString;

class ImageHeuristicScore : public HeuristicScore
{
public:
	ImageHeuristicScore(int score = 0);
	
	virtual int getNumericScore() const;
	
	virtual int getNumericSizeScore() const;
	
	void processPageHint(int hint) { m_score += hint; }
	
	void processSize(int width, int height); // use -1 for undefined
	
	void processUrls(URI const& base,
		URI const& img_abs, URI const& img_rel,
		URI const& link_abs, URI const& link_rel);
	
	void processLinkTarget(BString const& target);
	
	// are we inside <noscript>?
	void processNoscript(bool val);
	
	// does the image have the usemap attribute?
	void processAssociatedMap(bool val);
	
	// level 0: no script, 1: from script, 2: from script inside script, ...
	void processScriptLevel(int level, bool external);
	
	// was there a redirect when fetching an external script?
	void processScriptRedirect(bool val); 
	
	// were persistent cookies sent when fetching an external script?
	void processScriptPersistentCookies(bool val);
private:
	enum Flags {
		IMG_EXT_JPG        = 1 << 0,
		FROM_SCRIPT        = 1 << 1,
		IMG_PAGE_UNRELATED = 1 << 2,
		IMG_LINK_SAME_HOST = 1 << 3
	};
	
	int m_score;
	int m_sizeScore;
	int m_flags;
};

#endif
