/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "pch.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "OperationLog.h"
#include "EncOps.h"
#include "StringUtils.h"
#include <ace/config-lite.h>
#include <ace/Synch.h>
#include <ace/Singleton.h>
#include <glibmm/ustring.h>
#include <gtkmm/texttag.h>
#include <sstream>

using namespace std;

namespace GtkGUI
{

OperationLog::OperationLog()
{
	m_ptrMonospacedStyle = getTextBuffer()->create_tag();
	m_ptrMonospacedStyle->property_family() = "monospace";
}

OperationLog::~OperationLog()
{
}

OperationLog*
OperationLog::instance()
{
	return ACE_Singleton<OperationLog, ACE_Null_Mutex>::instance();
}

void
OperationLog::appendJsError(
	std::string const& msg, int lineno,
	std::string const& linebuf, int linepos)
{
	if (linebuf.empty()) {
		ostringstream strm;
		strm << '[' << lineno << "] " << msg;
		Glib::ustring text(EncOps::localeToUtf8(strm.str()));
		appendRecord(text, getWarningStyle());
		return;
	}
	
	char const* line_begin = StringUtils::ltrim(
		linebuf.c_str(), linebuf.c_str()+linebuf.size()
	);
	linepos -= line_begin - linebuf.c_str();
	char const* line_end = StringUtils::rtrim(
		line_begin, linebuf.c_str()+linebuf.size()
	);
	string line(line_begin, line_end);
	string pointer(linepos, ' ');
	pointer += '^';
	
	ostringstream strm1;
	strm1 << '[' << lineno << "] " << msg;
	ostringstream strm2;
	strm2 << "\n" << line << "\n" << pointer;
	Glib::ustring line1(EncOps::localeToUtf8(strm1.str()));
	Glib::ustring line2(EncOps::localeToUtf8(strm2.str()));
	appendRecord(line1, getWarningStyle());
	appendToLastRecord(line2, m_ptrMonospacedStyle);
}

} // namespace GtkGUI
